#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2018.  Cumulus Networks, Inc.
#

import os
import cumulus.platform
import cumulus.cpld
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx
import cumulus.platforms.mlnx_x86_MSN2100

# System LEDs
class MlxSN2010StatusLED(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLED):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4',
               'PSU1', 'PSU2',
               'Temp1', 'Temp2', 'Temp3', 'Temp4',
               'Temp5', 'Temp6', 'Temp7']
    led_location = ['led_status_green', 'led_status_red']

class MlxSN2010Psu1LED(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLED):
    name = 'Psu1'
    sources = ['PSU1']
    led_location = ['led_psu1_green', 'led_psu1_red']

class MlxSN2010Psu2LED(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLED):
    name = 'Psu2'
    sources = ['PSU2']
    led_location = ['led_psu2_green', 'led_psu2_red']

class MlxSN2010FanLED(cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLED):
    name = 'Fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4']
    led_location = ['led_fan_green', 'led_fan_red']

class MlxSN2010LEDs():
    units = (
        MlxSN2010StatusLED(),
        MlxSN2010Psu1LED(),
        MlxSN2010Psu2LED(),
        MlxSN2010FanLED(),
    )

class MlxSN2010SFP28(cumulus.platform.Spectrum25GPort):
    _set_led = '''
;;; MlxSN2010SFP28
'''
    def __init__(self, label, adj_label, phy_addr, hw_port):
        hw_port_name = 'xe%d' % hw_port

        cumulus.platform.Spectrum25GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_port=hw_port_name,
                        phy_addr=phy_addr)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_port, 'sfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2010QSFP28(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; MlxSN2010QSFP28
;;;  port LEDs controlled by HW
'''
    def __init__(self, label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(self, label=label,
                        adj_label=None, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2010Switch(cumulus.platform.SpectrumSwitch):
    _sfp28 = MlxSN2010SFP28
    _qsfp28 = MlxSN2010QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _sfp28(label="1",  adj_label=None, phy_addr=0,     hw_port=45),
        _sfp28(label="2",  adj_label=None, phy_addr=1,     hw_port=46),
        _sfp28(label="3",  adj_label=None, phy_addr=2,     hw_port=41),
        _sfp28(label="4",  adj_label=None, phy_addr=3,     hw_port=42),
        _sfp28(label="5",  adj_label=None, phy_addr=4,     hw_port=43),
        _sfp28(label="6",  adj_label=None, phy_addr=5,     hw_port=44),
        _sfp28(label="7",  adj_label=None, phy_addr=6,     hw_port=37),
        _sfp28(label="8",  adj_label=None, phy_addr=7,     hw_port=38),
        _sfp28(label="9",  adj_label=None, phy_addr=8,     hw_port=39),
        _sfp28(label="10", adj_label=None, phy_addr=9,     hw_port=40),
        _sfp28(label="11", adj_label=None, phy_addr=10,    hw_port=33),
        _sfp28(label="12", adj_label=None, phy_addr=11,    hw_port=34),
        _sfp28(label="13", adj_label=None, phy_addr=12,    hw_port=35),
        _sfp28(label="14", adj_label=None, phy_addr=13,    hw_port=36),
        _sfp28(label="15", adj_label=None, phy_addr=14,    hw_port=1),
        _sfp28(label="16", adj_label=None, phy_addr=15,    hw_port=2),
        _sfp28(label="17", adj_label=None, phy_addr=16,    hw_port=3),
        _sfp28(label="18", adj_label=None, phy_addr=17,    hw_port=4),
        _qsfp28(label="19", phy_addrs=(18,18,18,18), hw_ports=(9,10,11,12)),
        _qsfp28(label="20", phy_addrs=(19,19,19,19), hw_ports=(5,6,7,8)),
        _qsfp28(label="21", phy_addrs=(20,20,20,20), hw_ports=(17,18,19,20)),
        _qsfp28(label="22", phy_addrs=(21,21,21,21), hw_ports=(13,14,15,16)),
    )

class MlxSN2010(cumulus.platform.Platform):
    name = 'mlnx,msn2010'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN2010Switch(cumulus.platform.SpectrumChip())
    cpld = cumulus.platforms.mlnx_SN2xxx.Mlx2xxxCPLD()
    sensors = cumulus.platforms.mlnx_x86_MSN2100.MlxSN2100Sensors()
    pwms = None
    statusleds = MlxSN2010LEDs()

Platform = MlxSN2010
