#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN4xxx
from cumulus.chmgr_port_list import chmgr_port_list_num_of_front_panel_ports 
from cumulus.chmgr_port_list import chmgr_port_list_switch_port_cfg 

SN4800_MAX_LC_SLOTS = 8

class MlxSN4800QSFP28(cumulus.platform.Buffalo100GPort):
    _set_led = '''
;;; MlxSN4800QSFP28
;;; Two bits per port: yellow(active high), green(active high)
''' 

    def __init__(self, local_port_cfg, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        try:
            num_of_split_ports = len(local_port_cfg[label])
        except KeyError:
            num_of_split_ports = 0
        if num_of_split_ports != 0:
            hw_ports = []
            for split_no in range(num_of_split_ports):
                hw_ports.append((local_port_cfg[label])[split_no][1])
            hw_ports = tuple(hw_ports)

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Buffalo100GPort.__init__(
            self,
            label=label,
            adj_label=adj_label,
            hw_ports=tuple(hw_port_names),
            phy_addrs=phy_addrs,
            blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN4800QSFP56(cumulus.platform.Buffalo200GPort):
    _set_led = '''
;;; MlxSN4800QSFP56
;;; Two bits per port: yellow(active high), green(active high)
''' 

    def __init__(self, local_port_cfg, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        try:
            num_of_split_ports = len(local_port_cfg[label])
        except KeyError:
            num_of_split_ports = 0
        if num_of_split_ports != 0:
            hw_ports = []
            for split_no in range(num_of_split_ports):
                hw_ports.append((local_port_cfg[label])[split_no][1])
            hw_ports = tuple(hw_ports)

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Buffalo200GPort.__init__(
            self,
            label=label,
            adj_label=adj_label,
            hw_ports=tuple(hw_port_names),
            phy_addrs=phy_addrs,
            blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp56')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN4800QSFPDD(cumulus.platform.Buffalo400GPort):
    _set_led = '''
;;; MlxSN4800QSFPDD
;;; Two bits per port: yellow(active high), green(active high)
'''

    def __init__(self, local_port_cfg, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        try:
            num_of_split_ports = len(local_port_cfg[label])
        except KeyError:
            num_of_split_ports = 0
        if num_of_split_ports != 0:
            hw_ports = []
            for split_no in range(num_of_split_ports):
                hw_ports.append((local_port_cfg[label])[split_no][1])
            hw_ports = tuple(hw_ports)

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Buffalo400GPort.__init__(
            self,
            label=label,
            adj_label=adj_label,
            hw_ports=tuple(hw_port_names),
            phy_addrs=phy_addrs,
            blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp56')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

def Mlnx4800_lc_type():
    lc_map = {}

    cardsconf = open('/etc/cumulus/cards.conf', 'r')
    for line in cardsconf.readlines():
        line = line.strip()

        if '#' in line:
            continue
        slotinfo = line.split()
        lc_map[slotinfo[0]] = slotinfo[3]
    cardsconf.close()

    return lc_map

class MlxSN4800Switch(cumulus.platform.SpectrumSwitch):

    _sfp = None
    _qsfp28 = MlxSN4800QSFP28
    _qsfp56 = MlxSN4800QSFP56
    _qsfpdd = MlxSN4800QSFPDD

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False
    total_num_fp_ports = chmgr_port_list_num_of_front_panel_ports()
    local_port_cfg = chmgr_port_list_switch_port_cfg(total_num_fp_ports)

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can
    #    be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp28(local_port_cfg, label="nve", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(  0,   0,   0,   0,   0,   0,   0,   0)),
    )

    #TODO: rectify the lane mapping and phy_addrs below
    ports_slot1_400G = (
        _qsfpdd(local_port_cfg, label="m1p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(105, 106, 107, 108, 109, 110, 111, 112)),
        _qsfpdd(local_port_cfg, label="m1p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(109, 110, 111, 112),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m1p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 97,  98,  99, 100, 101, 102, 103, 104)),
        _qsfpdd(local_port_cfg, label="m1p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(101, 102, 103, 104),blocked_adj_split=8),
    )
    ports_slot2_400G = (
        _qsfpdd(local_port_cfg, label="m2p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(121, 122, 123, 124, 125, 126, 127, 128)),
        _qsfpdd(local_port_cfg, label="m2p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(125, 126, 127, 128),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m2p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=(113, 114, 115, 116, 117, 118, 119, 120)),
        _qsfpdd(local_port_cfg, label="m2p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(117, 118, 119, 120),blocked_adj_split=8),
    )
    ports_slot3_400G = (
        _qsfpdd(local_port_cfg, label="m3p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 89,  90,  91,  92,  93,  94,  95,  96)),
        _qsfpdd(local_port_cfg, label="m3p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 93,  94,  95,  96),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m3p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 81,  82,  83,  84,  85,  86,  87,  88)),
        _qsfpdd(local_port_cfg, label="m3p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 85,  86,  87,  88),blocked_adj_split=8),
    )
    ports_slot4_400G = (
        _qsfpdd(local_port_cfg, label="m4p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 73,  74,  75,  76,  77,  78,  79,  80)),
        _qsfpdd(local_port_cfg, label="m4p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 77,  78,  79,  80),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m4p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 65,  66,  67,  68,  69,  70,  71,  72)),
        _qsfpdd(local_port_cfg, label="m4p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 69,  70,  71,  72),blocked_adj_split=8),
    )
    ports_slot5_400G = (
        _qsfpdd(local_port_cfg, label="m5p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(  5,   6,   7,   8),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m5p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(  1,   2,   3,   4,   5,   6,   7,   8)),
        _qsfpdd(local_port_cfg, label="m5p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 13,  14,  15,  16),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m5p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(  9,  10,  11,  12,  13,  14,  15,  16)),
    )
    ports_slot6_400G = (
        _qsfpdd(local_port_cfg, label="m6p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 21,  22,  23,  24),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m6p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 17,  18,  19,  20,  21,  22,  23,  24)),
        _qsfpdd(local_port_cfg, label="m6p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 29,  30,  31,  32),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m6p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 25,  26,  27,  28,  29,  30,  31,  32)),
    )
    ports_slot7_400G = (
        _qsfpdd(local_port_cfg, label="m7p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 53,  54,  55,  56),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m7p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 49,  50,  51,  52,  53,  54,  55,  56)),
        _qsfpdd(local_port_cfg, label="m7p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 61,  62,  63,  64),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m7p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 57,  58,  59,  60,  61,  62,  63,  64)),
    )
    ports_slot8_400G = (
        _qsfpdd(local_port_cfg, label="m8p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 37,  38,  39,  40),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m8p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 33,  34,  35,  36,  37,  38,  39,  40)),
        _qsfpdd(local_port_cfg, label="m8p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 45,  46,  47,  48),blocked_adj_split=8),
        _qsfpdd(local_port_cfg, label="m8p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 41,  42,  43,  44,  45,  46,  47,  48)),
    )

    ports_slot1_200G = (
        _qsfp56(local_port_cfg, label="m1p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(105, 106, 107, 108, 109, 110, 111, 112)),
        _qsfp56(local_port_cfg, label="m1p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(109, 110, 111, 112),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m1p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 97,  98,  99, 100, 101, 102, 103, 104)),
        _qsfp56(local_port_cfg, label="m1p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(101, 102, 103, 104),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m1p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=(105, 106, 107, 108, 109, 110, 111, 112)),
        _qsfp56(local_port_cfg, label="m1p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=(109, 110, 111, 112),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m1p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 97,  98,  99, 100, 101, 102, 103, 104)),
        _qsfp56(local_port_cfg, label="m1p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=(101, 102, 103, 104),blocked_adj_split=8),
    )
    ports_slot2_200G = (
        _qsfp56(local_port_cfg, label="m2p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(121, 122, 123, 124, 125, 126, 127, 128)),
        _qsfp56(local_port_cfg, label="m2p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(125, 126, 127, 128),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m2p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=(113, 114, 115, 116, 117, 118, 119, 120)),
        _qsfp56(local_port_cfg, label="m2p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(117, 118, 119, 120),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m2p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=(121, 122, 123, 124, 125, 126, 127, 128)),
        _qsfp56(local_port_cfg, label="m2p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=(125, 126, 127, 128),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m2p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=(113, 114, 115, 116, 117, 118, 119, 120)),
        _qsfp56(local_port_cfg, label="m2p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=(117, 118, 119, 120),blocked_adj_split=8),
    )
    ports_slot3_200G = (
        _qsfp56(local_port_cfg, label="m3p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 89,  90,  91,  92,  93,  94,  95,  96)),
        _qsfp56(local_port_cfg, label="m3p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 93,  94,  95,  96),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m3p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 81,  82,  83,  84,  85,  86,  87,  88)),
        _qsfp56(local_port_cfg, label="m3p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 85,  86,  87,  88),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m3p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 89,  90,  91,  92,  93,  94,  95,  96)),
        _qsfp56(local_port_cfg, label="m3p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 93,  94,  95,  96),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m3p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 81,  82,  83,  84,  85,  86,  87,  88)),
        _qsfp56(local_port_cfg, label="m3p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 85,  86,  87,  88),blocked_adj_split=8),
    )
    ports_slot4_200G = (
        _qsfp56(local_port_cfg, label="m4p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 73,  74,  75,  76,  77,  78,  79,  80)),
        _qsfp56(local_port_cfg, label="m4p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 77,  78,  79,  80),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m4p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 65,  66,  67,  68,  69,  70,  71,  72)),
        _qsfp56(local_port_cfg, label="m4p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 69,  70,  71,  72),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m4p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 73,  74,  75,  76,  77,  78,  79,  80)),
        _qsfp56(local_port_cfg, label="m4p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 77,  78,  79,  80),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m4p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 65,  66,  67,  68,  69,  70,  71,  72)),
        _qsfp56(local_port_cfg, label="m4p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 69,  70,  71,  72),blocked_adj_split=8),
    )
    ports_slot5_200G = (
        _qsfp56(local_port_cfg, label="m5p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(  5,   6,   7,   8),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m5p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(  1,   2,   3,   4,   5,   6,   7,   8)),
        _qsfp56(local_port_cfg, label="m5p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 13,  14,  15,  16),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m5p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(  9,  10,  11,  12,  13,  14,  15,  16)),
        _qsfp56(local_port_cfg, label="m5p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=(  5,   6,   7,   8),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m5p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=(  1,   2,   3,   4,   5,   6,   7,   8)),
        _qsfp56(local_port_cfg, label="m5p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 13,  14,  15,  16),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m5p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=(  9,  10,  11,  12,  13,  14,  15,  16)),
    )
    ports_slot6_200G = (
        _qsfp56(local_port_cfg, label="m6p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 21,  22,  23,  24),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m6p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 17,  18,  19,  20,  21,  22,  23,  24)),
        _qsfp56(local_port_cfg, label="m6p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 29,  30,  31,  32),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m6p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 25,  26,  27,  28,  29,  30,  31,  32)),
        _qsfp56(local_port_cfg, label="m6p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 21,  22,  23,  24),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m6p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 17,  18,  19,  20,  21,  22,  23,  24)),
        _qsfp56(local_port_cfg, label="m6p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 29,  30,  31,  32),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m6p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 25,  26,  27,  28,  29,  30,  31,  32)),
    )
    ports_slot7_200G = (
        _qsfp56(local_port_cfg, label="m7p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 53,  54,  55,  56),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m7p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 49,  50,  51,  52,  53,  54,  55,  56)),
        _qsfp56(local_port_cfg, label="m7p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 61,  62,  63,  64),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m7p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 57,  58,  59,  60,  61,  62,  63,  64)),
        _qsfp56(local_port_cfg, label="m7p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 53,  54,  55,  56),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m7p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 49,  50,  51,  52,  53,  54,  55,  56)),
        _qsfp56(local_port_cfg, label="m7p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 61,  62,  63,  64),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m7p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 57,  58,  59,  60,  61,  62,  63,  64)),
    )
    ports_slot8_200G = (
        _qsfp56(local_port_cfg, label="m8p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 37,  38,  39,  40),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m8p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 33,  34,  35,  36,  37,  38,  39,  40)),
        _qsfp56(local_port_cfg, label="m8p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 45,  46,  47,  48),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m8p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 41,  42,  43,  44,  45,  46,  47,  48)),
        _qsfp56(local_port_cfg, label="m8p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 37,  38,  39,  40),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m8p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 33,  34,  35,  36,  37,  38,  39,  40)),
        _qsfp56(local_port_cfg, label="m8p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 45,  46,  47,  48),blocked_adj_split=8),
        _qsfp56(local_port_cfg, label="m8p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 41,  42,  43,  44,  45,  46,  47,  48)),
    )

    ports_slot1_100G = (
        _qsfp28(local_port_cfg, label="m1p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(105, 106, 107, 108, 109, 110, 111, 112)),
        _qsfp28(local_port_cfg, label="m1p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(109, 110, 111, 112),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m1p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 97,  98,  99, 100, 101, 102, 103, 104)),
        _qsfp28(local_port_cfg, label="m1p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(101, 102, 103, 104),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m1p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=(105, 106, 107, 108, 109, 110, 111, 112)),
        _qsfp28(local_port_cfg, label="m1p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=(109, 110, 111, 112),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m1p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 97,  98,  99, 100, 101, 102, 103, 104)),
        _qsfp28(local_port_cfg, label="m1p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=(101, 102, 103, 104),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m1p9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8,  8,  8,  8,  8), hw_ports=(105, 106, 107, 108, 109, 110, 111, 112)),
        _qsfp28(local_port_cfg, label="m1p10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9,  9,  9,  9,  9), hw_ports=(109, 110, 111, 112),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m1p11", adj_label=None,
              phy_addrs=(10, 10, 10, 10, 10, 10, 10, 10), hw_ports=( 97,  98,  99, 100, 101, 102, 103, 104)),
        _qsfp28(local_port_cfg, label="m1p12", adj_label=None,
              phy_addrs=(11, 11, 11, 11, 11, 11, 11, 11), hw_ports=(101, 102, 103, 104),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m1p13", adj_label=None,
              phy_addrs=(12, 12, 12, 12, 12, 12, 12, 12), hw_ports=(105, 106, 107, 108, 109, 110, 111, 112)),
        _qsfp28(local_port_cfg, label="m1p14", adj_label=None,
              phy_addrs=(13, 13, 13, 13, 13, 13, 13, 13), hw_ports=(109, 110, 111, 112),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m1p15", adj_label=None,
              phy_addrs=(14, 14, 14, 14, 14, 14, 14, 14), hw_ports=( 97,  98,  99, 100, 101, 102, 103, 104)),
        _qsfp28(local_port_cfg, label="m1p16", adj_label=None,
              phy_addrs=(15, 15, 15, 15, 15, 15, 15, 15), hw_ports=(101, 102, 103, 104),blocked_adj_split=8),
    )
    ports_slot2_100G = (
        _qsfp28(local_port_cfg, label="m2p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(121, 122, 123, 124, 125, 126, 127, 128)),
        _qsfp28(local_port_cfg, label="m2p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(125, 126, 127, 128),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m2p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=(113, 114, 115, 116, 117, 118, 119, 120)),
        _qsfp28(local_port_cfg, label="m2p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(117, 118, 119, 120),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m2p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=(121, 122, 123, 124, 125, 126, 127, 128)),
        _qsfp28(local_port_cfg, label="m2p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=(125, 126, 127, 128),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m2p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=(113, 114, 115, 116, 117, 118, 119, 120)),
        _qsfp28(local_port_cfg, label="m2p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=(117, 118, 119, 120),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m2p9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8,  8,  8,  8,  8), hw_ports=(121, 122, 123, 124, 125, 126, 127, 128)),
        _qsfp28(local_port_cfg, label="m2p10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9,  9,  9,  9,  9), hw_ports=(125, 126, 127, 128),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m2p11", adj_label=None,
              phy_addrs=(10, 10, 10, 10, 10, 10, 10, 10), hw_ports=(113, 114, 115, 116, 117, 118, 119, 120)),
        _qsfp28(local_port_cfg, label="m2p12", adj_label=None,
              phy_addrs=(11, 11, 11, 11, 11, 11, 11, 11), hw_ports=(117, 118, 119, 120),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m2p13", adj_label=None,
              phy_addrs=(12, 12, 12, 12, 12, 12, 12, 12), hw_ports=(121, 122, 123, 124, 125, 126, 127, 128)),
        _qsfp28(local_port_cfg, label="m2p14", adj_label=None,
              phy_addrs=(13, 13, 13, 13, 13, 13, 13, 13), hw_ports=(125, 126, 127, 128),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m2p15", adj_label=None,
              phy_addrs=(14, 14, 14, 14, 14, 14, 14, 14), hw_ports=(113, 114, 115, 116, 117, 118, 119, 120)),
        _qsfp28(local_port_cfg, label="m2p16", adj_label=None,
              phy_addrs=(15, 15, 15, 15, 15, 15, 15, 15), hw_ports=(117, 118, 119, 120),blocked_adj_split=8),
    )
    ports_slot3_100G = (
        _qsfp28(local_port_cfg, label="m3p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 89,  90,  91,  92,  93,  94,  95,  96)),
        _qsfp28(local_port_cfg, label="m3p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 93,  94,  95,  96),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m3p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 81,  82,  83,  84,  85,  86,  87,  88)),
        _qsfp28(local_port_cfg, label="m3p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 85,  86,  87,  88),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m3p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 89,  90,  91,  92,  93,  94,  95,  96)),
        _qsfp28(local_port_cfg, label="m3p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 93,  94,  95,  96),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m3p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 81,  82,  83,  84,  85,  86,  87,  88)),
        _qsfp28(local_port_cfg, label="m3p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 85,  86,  87,  88),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m3p9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8,  8,  8,  8,  8), hw_ports=( 89,  90,  91,  92,  93,  94,  95,  96)),
        _qsfp28(local_port_cfg, label="m3p10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9,  9,  9,  9,  9), hw_ports=( 93,  94,  95,  96),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m3p11", adj_label=None,
              phy_addrs=(10, 10, 10, 10, 10, 10, 10, 10), hw_ports=( 81,  82,  83,  84,  85,  86,  87,  88)),
        _qsfp28(local_port_cfg, label="m3p12", adj_label=None,
              phy_addrs=(11, 11, 11, 11, 11, 11, 11, 11), hw_ports=( 85,  86,  87,  88),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m3p13", adj_label=None,
              phy_addrs=(12, 12, 12, 12, 12, 12, 12, 12), hw_ports=( 89,  90,  91,  92,  93,  94,  95,  96)),
        _qsfp28(local_port_cfg, label="m3p14", adj_label=None,
              phy_addrs=(13, 13, 13, 13, 13, 13, 13, 13), hw_ports=( 93,  94,  95,  96),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m3p15", adj_label=None,
              phy_addrs=(14, 14, 14, 14, 14, 14, 14, 14), hw_ports=( 81,  82,  83,  84,  85,  86,  87,  88)),
        _qsfp28(local_port_cfg, label="m3p16", adj_label=None,
              phy_addrs=(15, 15, 15, 15, 15, 15, 15, 15), hw_ports=( 85,  86,  87,  88),blocked_adj_split=8),
    )
    ports_slot4_100G = (
        _qsfp28(local_port_cfg, label="m4p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 73,  74,  75,  76,  77,  78,  79,  80)),
        _qsfp28(local_port_cfg, label="m4p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 77,  78,  79,  80),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m4p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 65,  66,  67,  68,  69,  70,  71,  72)),
        _qsfp28(local_port_cfg, label="m4p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 69,  70,  71,  72),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m4p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 73,  74,  75,  76,  77,  78,  79,  80)),
        _qsfp28(local_port_cfg, label="m4p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 77,  78,  79,  80),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m4p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 65,  66,  67,  68,  69,  70,  71,  72)),
        _qsfp28(local_port_cfg, label="m4p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 69,  70,  71,  72),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m4p9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8,  8,  8,  8,  8), hw_ports=( 73,  74,  75,  76,  77,  78,  79,  80)),
        _qsfp28(local_port_cfg, label="m4p10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9,  9,  9,  9,  9), hw_ports=( 77,  78,  79,  80),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m4p11", adj_label=None,
              phy_addrs=(10, 10, 10, 10, 10, 10, 10, 10), hw_ports=( 65,  66,  67,  68,  69,  70,  71,  72)),
        _qsfp28(local_port_cfg, label="m4p12", adj_label=None,
              phy_addrs=(11, 11, 11, 11, 11, 11, 11, 11), hw_ports=( 69,  70,  71,  72),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m4p13", adj_label=None,
              phy_addrs=(12, 12, 12, 12, 12, 12, 12, 12), hw_ports=( 73,  74,  75,  76,  77,  78,  79,  80)),
        _qsfp28(local_port_cfg, label="m4p14", adj_label=None,
              phy_addrs=(13, 13, 13, 13, 13, 13, 13, 13), hw_ports=( 77,  78,  79,  80),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m4p15", adj_label=None,
              phy_addrs=(14, 14, 14, 14, 14, 14, 14, 14), hw_ports=( 65,  66,  67,  68,  69,  70,  71,  72)),
        _qsfp28(local_port_cfg, label="m4p16", adj_label=None,
              phy_addrs=(15, 15, 15, 15, 15, 15, 15, 15), hw_ports=( 69,  70,  71,  72),blocked_adj_split=8),
    )
    ports_slot5_100G = (
        _qsfp28(local_port_cfg, label="m5p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=(  5,   6,   7,   8),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m5p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=(  1,   2,   3,   4,   5,   6,   7,   8)),
        _qsfp28(local_port_cfg, label="m5p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 13,  14,  15,  16),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m5p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=(  9,  10,  11,  12,  13,  14,  15,  16)),
        _qsfp28(local_port_cfg, label="m5p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=(  5,   6,   7,   8),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m5p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=(  1,   2,   3,   4,   5,   6,   7,   8)),
        _qsfp28(local_port_cfg, label="m5p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 13,  14,  15,  16),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m5p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=(  9,  10,  11,  12,  13,  14,  15,  16)),
        _qsfp28(local_port_cfg, label="m5p9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8,  8,  8,  8,  8), hw_ports=(  5,   6,   7,   8),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m5p10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9,  9,  9,  9,  9), hw_ports=(  1,   2,   3,   4,   5,   6,   7,   8)),
        _qsfp28(local_port_cfg, label="m5p11", adj_label=None,
              phy_addrs=(10, 10, 10, 10, 10, 10, 10, 10), hw_ports=( 13,  14,  15,  16),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m5p12", adj_label=None,
              phy_addrs=(11, 11, 11, 11, 11, 11, 11, 11), hw_ports=(  9,  10,  11,  12,  13,  14,  15,  16)),
        _qsfp28(local_port_cfg, label="m5p13", adj_label=None,
              phy_addrs=(12, 12, 12, 12, 12, 12, 12, 12), hw_ports=(  5,   6,   7,   8),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m5p14", adj_label=None,
              phy_addrs=(13, 13, 13, 13, 13, 13, 13, 13), hw_ports=(  1,   2,   3,   4,   5,   6,   7,   8)),
        _qsfp28(local_port_cfg, label="m5p15", adj_label=None,
              phy_addrs=(14, 14, 14, 14, 14, 14, 14, 14), hw_ports=( 13,  14,  15,  16),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m5p16", adj_label=None,
              phy_addrs=(15, 15, 15, 15, 15, 15, 15, 15), hw_ports=(  9,  10,  11,  12,  13,  14,  15,  16)),
    )
    ports_slot6_100G = (
        _qsfp28(local_port_cfg, label="m6p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 21,  22,  23,  24),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m6p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 17,  18,  19,  20,  21,  22,  23,  24)),
        _qsfp28(local_port_cfg, label="m6p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 29,  30,  31,  32),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m6p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 25,  26,  27,  28,  29,  30,  31,  32)),
        _qsfp28(local_port_cfg, label="m6p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 21,  22,  23,  24),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m6p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 17,  18,  19,  20,  21,  22,  23,  24)),
        _qsfp28(local_port_cfg, label="m6p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 29,  30,  31,  32),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m6p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 25,  26,  27,  28,  29,  30,  31,  32)),
        _qsfp28(local_port_cfg, label="m6p9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8,  8,  8,  8,  8), hw_ports=( 21,  22,  23,  24),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m6p10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9,  9,  9,  9,  9), hw_ports=( 17,  18,  19,  20,  21,  22,  23,  24)),
        _qsfp28(local_port_cfg, label="m6p11", adj_label=None,
              phy_addrs=(10, 10, 10, 10, 10, 10, 10, 10), hw_ports=( 29,  30,  31,  32),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m6p12", adj_label=None,
              phy_addrs=(11, 11, 11, 11, 11, 11, 11, 11), hw_ports=( 25,  26,  27,  28,  29,  30,  31,  32)),
        _qsfp28(local_port_cfg, label="m6p13", adj_label=None,
              phy_addrs=(12, 12, 12, 12, 12, 12, 12, 12), hw_ports=( 21,  22,  23,  24),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m6p14", adj_label=None,
              phy_addrs=(13, 13, 13, 13, 13, 13, 13, 13), hw_ports=( 17,  18,  19,  20,  21,  22,  23,  24)),
        _qsfp28(local_port_cfg, label="m6p15", adj_label=None,
              phy_addrs=(14, 14, 14, 14, 14, 14, 14, 14), hw_ports=( 29,  30,  31,  32),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m6p16", adj_label=None,
              phy_addrs=(15, 15, 15, 15, 15, 15, 15, 15), hw_ports=( 25,  26,  27,  28,  29,  30,  31,  32)),
    )
    ports_slot7_100G = (
        _qsfp28(local_port_cfg, label="m7p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 53,  54,  55,  56),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m7p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 49,  50,  51,  52,  53,  54,  55,  56)),
        _qsfp28(local_port_cfg, label="m7p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 61,  62,  63,  64),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m7p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 57,  58,  59,  60,  61,  62,  63,  64)),
        _qsfp28(local_port_cfg, label="m7p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 53,  54,  55,  56),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m7p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 49,  50,  51,  52,  53,  54,  55,  56)),
        _qsfp28(local_port_cfg, label="m7p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 61,  62,  63,  64),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m7p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 57,  58,  59,  60,  61,  62,  63,  64)),
        _qsfp28(local_port_cfg, label="m7p9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8,  8,  8,  8,  8), hw_ports=( 53,  54,  55,  56),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m7p10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9,  9,  9,  9,  9), hw_ports=( 49,  50,  51,  52,  53,  54,  55,  56)),
        _qsfp28(local_port_cfg, label="m7p11", adj_label=None,
              phy_addrs=(10, 10, 10, 10, 10, 10, 10, 10), hw_ports=( 61,  62,  63,  64),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m7p12", adj_label=None,
              phy_addrs=(11, 11, 11, 11, 11, 11, 11, 11), hw_ports=( 57,  58,  59,  60,  61,  62,  63,  64)),
        _qsfp28(local_port_cfg, label="m7p13", adj_label=None,
              phy_addrs=(12, 12, 12, 12, 12, 12, 12, 12), hw_ports=( 53,  54,  55,  56),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m7p14", adj_label=None,
              phy_addrs=(13, 13, 13, 13, 13, 13, 13, 13), hw_ports=( 49,  50,  51,  52,  53,  54,  55,  56)),
        _qsfp28(local_port_cfg, label="m7p15", adj_label=None,
              phy_addrs=(14, 14, 14, 14, 14, 14, 14, 14), hw_ports=( 61,  62,  63,  64),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m7p16", adj_label=None,
              phy_addrs=(15, 15, 15, 15, 15, 15, 15, 15), hw_ports=( 57,  58,  59,  60,  61,  62,  63,  64)),
    )
    ports_slot8_100G = (
        _qsfp28(local_port_cfg, label="m8p1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0,  0,  0,  0,  0), hw_ports=( 37,  38,  39,  40),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m8p2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1,  1,  1,  1,  1), hw_ports=( 33,  34,  35,  36,  37,  38,  39,  40)),
        _qsfp28(local_port_cfg, label="m8p3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2,  2,  2,  2,  2), hw_ports=( 45,  46,  47,  48),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m8p4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3,  3,  3,  3,  3), hw_ports=( 41,  42,  43,  44,  45,  46,  47,  48)),
        _qsfp28(local_port_cfg, label="m8p5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4,  4,  4,  4,  4), hw_ports=( 37,  38,  39,  40),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m8p6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5,  5,  5,  5,  5), hw_ports=( 33,  34,  35,  36,  37,  38,  39,  40)),
        _qsfp28(local_port_cfg, label="m8p7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6,  6,  6,  6,  6), hw_ports=( 45,  46,  47,  48),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m8p8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7,  7,  7,  7,  7), hw_ports=( 41,  42,  43,  44,  45,  46,  47,  48)),
        _qsfp28(local_port_cfg, label="m8p9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8,  8,  8,  8,  8), hw_ports=( 37,  38,  39,  40),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m8p10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9,  9,  9,  9,  9), hw_ports=( 33,  34,  35,  36,  37,  38,  39,  40)),
        _qsfp28(local_port_cfg, label="m8p11", adj_label=None,
              phy_addrs=(10, 10, 10, 10, 10, 10, 10, 10), hw_ports=( 45,  46,  47,  48),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m8p12", adj_label=None,
              phy_addrs=(11, 11, 11, 11, 11, 11, 11, 11), hw_ports=( 41,  42,  43,  44,  45,  46,  47,  48)),
        _qsfp28(local_port_cfg, label="m8p13", adj_label=None,
              phy_addrs=(12, 12, 12, 12, 12, 12, 12, 12), hw_ports=( 37,  38,  39,  40),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m8p14", adj_label=None,
              phy_addrs=(13, 13, 13, 13, 13, 13, 13, 13), hw_ports=( 33,  34,  35,  36,  37,  38,  39,  40)),
        _qsfp28(local_port_cfg, label="m8p15", adj_label=None,
              phy_addrs=(14, 14, 14, 14, 14, 14, 14, 14), hw_ports=( 45,  46,  47,  48),blocked_adj_split=8),
        _qsfp28(local_port_cfg, label="m8p16", adj_label=None,
              phy_addrs=(15, 15, 15, 15, 15, 15, 15, 15), hw_ports=( 41,  42,  43,  44,  45,  46,  47,  48)),

    )

    lc_map = Mlnx4800_lc_type()
    slots = SN4800_MAX_LC_SLOTS
    for x in range(1, slots+1):
        slot_type = lc_map.get(str(x))
        if slot_type == '400G':
            if x == 1:
                ports = ports + ports_slot1_400G
            elif x == 2:
                ports = ports + ports_slot2_400G
            elif x == 3:
                ports = ports + ports_slot3_400G
            elif x == 4:
                ports = ports + ports_slot4_400G
            elif x == 5:
                ports = ports + ports_slot5_400G
            elif x == 6:
                ports = ports + ports_slot6_400G
            elif x == 7:
                ports = ports + ports_slot7_400G
            else:
                ports = ports + ports_slot8_400G
        elif slot_type == '200G':
            if x == 1:
                ports = ports + ports_slot1_200G
            elif x == 2:
                ports = ports + ports_slot2_200G
            elif x == 3:
                ports = ports + ports_slot3_200G
            elif x == 4:
                ports = ports + ports_slot4_200G
            elif x == 5:
                ports = ports + ports_slot5_200G
            elif x == 6:
                ports = ports + ports_slot6_200G
            elif x == 7:
                ports = ports + ports_slot7_200G
            else:
                ports = ports + ports_slot8_200G
        else:
            if x == 1:
                ports = ports + ports_slot1_100G
            elif x == 2:
                ports = ports + ports_slot2_100G
            elif x == 3:
                ports = ports + ports_slot3_100G
            elif x == 4:
                ports = ports + ports_slot4_100G
            elif x == 5:
                ports = ports + ports_slot5_100G
            elif x == 6:
                ports = ports + ports_slot6_100G
            elif x == 7:
                ports = ports + ports_slot7_100G
            else:
                ports = ports + ports_slot8_100G

# Base class for the Temp Sensors
class MlxSN4800Temp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 75
    temp_max = 95
    temp_crit = 115

class MlxSN4800LC1Temp_1(MlxSN4800Temp):
    name = 'LC-1 Temp1'
    description = 'Gearbox-1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox1_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox1/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox1/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc1/thermal/'

class MlxSN4800LC1Temp_2(MlxSN4800Temp):
    name = 'LC-1 Temp2'
    description = 'Gearbox-2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox2_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox2/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox2/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc1/thermal/'

class MlxSN4800LC1Temp_3(MlxSN4800Temp):
    name = 'LC-1 Temp3'
    description = 'Gearbox-3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox3_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox3/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox3/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc1/thermal/'

class MlxSN4800LC1Temp_4(MlxSN4800Temp):
    name = 'LC-1 Temp4'
    description = 'Gearbox-4 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox4_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox4/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox4/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc1/thermal/'

class MlxSN4800LC2Temp_1(MlxSN4800Temp):
    name = 'LC-2 Temp1'
    description = 'Gearbox-1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox1_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox1/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox1/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc2/thermal/'

class MlxSN4800LC2Temp_2(MlxSN4800Temp):
    name = 'LC-2 Temp2'
    description = 'Gearbox-2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox2_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox2/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox2/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc2/thermal/'

class MlxSN4800LC2Temp_3(MlxSN4800Temp):
    name = 'LC-2 Temp3'
    description = 'Gearbox-3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox3_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox3/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox3/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc2/thermal/'

class MlxSN4800LC2Temp_4(MlxSN4800Temp):
    name = 'LC-2 Temp4'
    description = 'Gearbox-4 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox4_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox4/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox4/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc2/thermal/'

class MlxSN4800LC3Temp_1(MlxSN4800Temp):
    name = 'LC-3 Temp1'
    description = 'Gearbox-1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox1_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox1/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox1/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc3/thermal/'

class MlxSN4800LC3Temp_2(MlxSN4800Temp):
    name = 'LC-3 Temp2'
    description = 'Gearbox-2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox2_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox2/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox2/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc3/thermal/'

class MlxSN4800LC3Temp_3(MlxSN4800Temp):
    name = 'LC-3 Temp3'
    description = 'Gearbox-3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox3_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox3/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox3/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc3/thermal/'

class MlxSN4800LC3Temp_4(MlxSN4800Temp):
    name = 'LC-3 Temp4'
    description = 'Gearbox-4 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox4_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox4/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox4/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc3/thermal/'

class MlxSN4800LC4Temp_1(MlxSN4800Temp):
    name = 'LC-4 Temp1'
    description = 'Gearbox-1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox1_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox1/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox1/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc4/thermal/'

class MlxSN4800LC4Temp_2(MlxSN4800Temp):
    name = 'LC-4 Temp2'
    description = 'Gearbox-2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox2_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox2/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox2/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc4/thermal/'

class MlxSN4800LC4Temp_3(MlxSN4800Temp):
    name = 'LC-4 Temp3'
    description = 'Gearbox-3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox3_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox3/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox3/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc4/thermal/'

class MlxSN4800LC4Temp_4(MlxSN4800Temp):
    name = 'LC-4 Temp4'
    description = 'Gearbox-4 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox4_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox4/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox4/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc4/thermal/'

class MlxSN4800LC5Temp_1(MlxSN4800Temp):
    name = 'LC-5 Temp1'
    description = 'Gearbox-1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox1_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox1/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox1/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc5/thermal/'

class MlxSN4800LC5Temp_2(MlxSN4800Temp):
    name = 'LC-5 Temp2'
    description = 'Gearbox-2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox2_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox2/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox2/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc5/thermal/'

class MlxSN4800LC5Temp_3(MlxSN4800Temp):
    name = 'LC-5 Temp3'
    description = 'Gearbox-3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox3_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox3/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox3/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc5/thermal/'

class MlxSN4800LC5Temp_4(MlxSN4800Temp):
    name = 'LC-5 Temp4'
    description = 'Gearbox-4 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox4_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox4/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox4/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc5/thermal/'

class MlxSN4800LC6Temp_1(MlxSN4800Temp):
    name = 'LC-6 Temp1'
    description = 'Gearbox-1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox1_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox1/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox1/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc6/thermal/'

class MlxSN4800LC6Temp_2(MlxSN4800Temp):
    name = 'LC-6 Temp2'
    description = 'Gearbox-2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox2_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox2/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox2/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc6/thermal/'

class MlxSN4800LC6Temp_3(MlxSN4800Temp):
    name = 'LC-6 Temp3'
    description = 'Gearbox-3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox3_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox3/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox3/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc6/thermal/'

class MlxSN4800LC6Temp_4(MlxSN4800Temp):
    name = 'LC-6 Temp4'
    description = 'Gearbox-4 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox4_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox4/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox4/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc6/thermal/'

class MlxSN4800LC7Temp_1(MlxSN4800Temp):
    name = 'LC-7 Temp1'
    description = 'Gearbox-1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox1_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox1/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox1/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc7/thermal/'

class MlxSN4800LC7Temp_2(MlxSN4800Temp):
    name = 'LC-7 Temp2'
    description = 'Gearbox-2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox2_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox2/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox2/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc7/thermal/'

class MlxSN4800LC7Temp_3(MlxSN4800Temp):
    name = 'LC-7 Temp3'
    description = 'Gearbox-3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox3_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox3/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox3/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc7/thermal/'

class MlxSN4800LC7Temp_4(MlxSN4800Temp):
    name = 'LC-7 Temp4'
    description = 'Gearbox-4 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox4_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox4/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox4/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc7/thermal/'

class MlxSN4800LC8Temp_1(MlxSN4800Temp):
    name = 'LC-8 Temp1'
    description = 'Gearbox-1 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox1_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox1/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox1/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc8/thermal/'

class MlxSN4800LC8Temp_2(MlxSN4800Temp):
    name = 'LC-8 Temp2'
    description = 'Gearbox-2 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox2_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox2/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox2/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc8/thermal/'

class MlxSN4800LC8Temp_3(MlxSN4800Temp):
    name = 'LC-8 Temp3'
    description = 'Gearbox-3 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox3_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox3/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox3/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc8/thermal/'

class MlxSN4800LC8Temp_4(MlxSN4800Temp):
    name = 'LC-8 Temp4'
    description = 'Gearbox-4 Sensor'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'gearbox4_temp_input'
    mlnx_hw_mgmt_temp_crit = 'mlxsw-gearbox4/temp_trip_crit'
    mlnx_hw_mgmt_temp_max = 'mlxsw-gearbox4/temp_trip_high'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'
    mlnx_hw_mgmt_path = '/run/hw-management/lc8/thermal/'


class MlxSN4800PsuLED(cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxLED):
    name = 'PSU'
    sources = ['PSU']
    led_location = ['led_psu_green', 'led_psu_orange']


class MlxSN4800StatusLED(cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxLED):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6',
               'PSU',
               'Temp1', 'Temp2',
               'Temp3', 'Temp4', 'Temp5', 'Temp6']
    led_location = ['led_status_green', 'led_status_orange']


class MlxSN4800LEDS():
    units = (
        MlxSN4800StatusLED(),
        MlxSN4800PsuLED(),
        cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan1LED(),
        cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan2LED(),
        cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan3LED(),
        cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan4LED(),
        cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan5LED(),
        cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan6LED(),
    )

class MlxSN4800Sensors():
    units = (cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_1(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_2(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_3(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_4(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_5(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxFan_6(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_1(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_2(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_3(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_4(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_5(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_6(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_7(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxTemp_8(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSU1(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSU2(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSU3(),
             cumulus.platforms.mlnx_SN4xxx.MlxSN4xxxPSU4()
             )
    slot1_gb = (MlxSN4800LC1Temp_1(),
                MlxSN4800LC1Temp_2(),
                MlxSN4800LC1Temp_3(),
                MlxSN4800LC1Temp_4(),
               )
    slot2_gb = (MlxSN4800LC2Temp_1(),
                MlxSN4800LC2Temp_2(),
                MlxSN4800LC2Temp_3(),
                MlxSN4800LC2Temp_4(),
               )
    slot3_gb = (MlxSN4800LC3Temp_1(),
                MlxSN4800LC3Temp_2(),
                MlxSN4800LC3Temp_3(),
                MlxSN4800LC3Temp_4(),
               )
    slot4_gb = (MlxSN4800LC4Temp_1(),
                MlxSN4800LC4Temp_2(),
                MlxSN4800LC4Temp_3(),
                MlxSN4800LC4Temp_4(),
               )
    slot5_gb = (MlxSN4800LC5Temp_1(),
                MlxSN4800LC5Temp_2(),
                MlxSN4800LC5Temp_3(),
                MlxSN4800LC5Temp_4(),
               )
    slot6_gb = (MlxSN4800LC6Temp_1(),
                MlxSN4800LC6Temp_2(),
                MlxSN4800LC6Temp_3(),
                MlxSN4800LC6Temp_4(),
               )
    slot7_gb = (MlxSN4800LC7Temp_1(),
                MlxSN4800LC7Temp_2(),
                MlxSN4800LC7Temp_3(),
                MlxSN4800LC7Temp_4(),
               )
    slot8_gb = (MlxSN4800LC8Temp_1(),
                MlxSN4800LC8Temp_2(),
                MlxSN4800LC8Temp_3(),
                MlxSN4800LC8Temp_4(),
               )

    lc_map = Mlnx4800_lc_type()
    slots = SN4800_MAX_LC_SLOTS
    for x in range(1, slots+1):
        slot_type = lc_map.get(str(x))
        if slot_type == '100G':
            if x == 1:
                units = units + slot1_gb
            elif x == 2:
                units = units + slot2_gb
            elif x == 3:
                units = units + slot3_gb
            elif x == 4:
                units = units + slot4_gb
            elif x == 5:
                units = units + slot5_gb
            elif x == 6:
                units = units + slot6_gb
            elif x == 7:
                units = units + slot7_gb
            elif x == 8:
                units = units + slot8_gb

class MlxSN4800(cumulus.platform.Platform):
    name = 'mlnx,msn4800'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN4800Switch(chip)
    cpld = cumulus.platforms.mlnx_SN4xxx.Mlx4xxxCPLD()
    sensors = MlxSN4800Sensors
    pwms = None
    statusleds = MlxSN4800LEDS
    use_rate_api = True
    modular = True
    slots = SN4800_MAX_LC_SLOTS

Platform = MlxSN4800
