#
# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.sensors
import cumulus.sfp


class MlxSimxSN4600QSFP28(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; MlxSimxSN4600QSFP28
;;; Two bits per port: yellow(active high), green(active high)
'''

    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(
            self,
            label=label,
            adj_label=adj_label,
            hw_ports=tuple(hw_port_names),
            phy_addrs=phy_addrs,
            blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls


class MlxSimxSN4600Switch(cumulus.platform.SpectrumSwitch):

    _sfp = None
    _qsfp = MlxSimxSN4600QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can
    #    be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp(label="nve", adj_label=None,
              phy_addrs=( 0,  0,  0,  0), hw_ports=(  0,   0)),
        _qsfp(label="1", adj_label=None,
              phy_addrs=( 0,  0,  0,  0), hw_ports=(105, 106, 107, 108)),
        _qsfp(label="2", adj_label=None,
              phy_addrs=( 1,  1,  1,  1), hw_ports=(107, 108), blocked_adj_split=4),
        _qsfp(label="3", adj_label=None,
              phy_addrs=( 2,  2,  2,  2), hw_ports=(109, 110, 111, 112)),
        _qsfp(label="4", adj_label=None,
              phy_addrs=( 3,  3,  3,  3), hw_ports=(111, 112), blocked_adj_split=4),
        _qsfp(label="5", adj_label=None,
              phy_addrs=( 4,  4,  4,  4), hw_ports=( 97,  98, 99, 100)),
        _qsfp(label="6", adj_label=None,
              phy_addrs=( 5,  5,  5,  5), hw_ports=( 99, 100), blocked_adj_split=4),
        _qsfp(label="7", adj_label=None,
              phy_addrs=( 6,  6,  6,  6), hw_ports=(101, 102, 103, 104)),
        _qsfp(label="8", adj_label=None,
              phy_addrs=( 7,  7,  7,  7), hw_ports=(103, 104), blocked_adj_split=4),
        _qsfp(label="9", adj_label=None,
              phy_addrs=( 8,  8,  8,  8), hw_ports=(121, 122, 123, 124)),
        _qsfp(label="10", adj_label=None,
              phy_addrs=( 9,  9,  9,  9), hw_ports=(123, 124), blocked_adj_split=4),
        _qsfp(label="11", adj_label=None,
              phy_addrs=(10, 10, 10, 10), hw_ports=(125, 126, 127, 128)),
        _qsfp(label="12", adj_label=None,
              phy_addrs=(11, 11, 11, 11), hw_ports=(127, 128), blocked_adj_split=4),
        _qsfp(label="13", adj_label=None,
              phy_addrs=(12, 12, 12, 12), hw_ports=(113, 114, 115, 116)),
        _qsfp(label="14", adj_label=None,
              phy_addrs=(13, 13, 13, 13), hw_ports=(115, 116), blocked_adj_split=4),
        _qsfp(label="15", adj_label=None,
              phy_addrs=(14, 14, 14, 14), hw_ports=(117, 118, 119, 120)),
        _qsfp(label="16", adj_label=None,
              phy_addrs=(15, 15, 15, 15), hw_ports=(119, 120), blocked_adj_split=4),
        _qsfp(label="17", adj_label=None,
              phy_addrs=(16, 16, 16, 16), hw_ports=( 89,  90,  91,  92)),
        _qsfp(label="18", adj_label=None,
              phy_addrs=(17, 17, 17, 17), hw_ports=( 91,  92), blocked_adj_split=4),
        _qsfp(label="19", adj_label=None,
              phy_addrs=(18, 18, 18, 18), hw_ports=( 93,  94,  95,  96)),
        _qsfp(label="20", adj_label=None,
              phy_addrs=(19, 19, 19, 19), hw_ports=( 95,  96), blocked_adj_split=4),
        _qsfp(label="21", adj_label=None,
              phy_addrs=(20, 20, 20, 20), hw_ports=( 81,  82,  83,  84)),
        _qsfp(label="22", adj_label=None,
              phy_addrs=(21, 21, 21, 21), hw_ports=( 83,  84), blocked_adj_split=4),
        _qsfp(label="23", adj_label=None,
              phy_addrs=(22, 22, 22, 22), hw_ports=( 85,  86,  87,  88)),
        _qsfp(label="24", adj_label=None,
              phy_addrs=(23, 23, 23, 23), hw_ports=( 87,  88), blocked_adj_split=4),
        _qsfp(label="25", adj_label=None,
              phy_addrs=(24, 24, 24, 24), hw_ports=( 73,  74,  75,  76)),
        _qsfp(label="26", adj_label=None,
              phy_addrs=(25, 25, 25, 25), hw_ports=( 75,  76), blocked_adj_split=4),
        _qsfp(label="27", adj_label=None,
              phy_addrs=(26, 26, 26, 26), hw_ports=( 77,  78,  79,  80)),
        _qsfp(label="28", adj_label=None,
              phy_addrs=(27, 27, 27, 27), hw_ports=( 79,  80), blocked_adj_split=4),
        _qsfp(label="29", adj_label=None,
              phy_addrs=(28, 28, 28, 28), hw_ports=( 65,  66,  67,  68)),
        _qsfp(label="30", adj_label=None,
              phy_addrs=(29, 29, 29, 29), hw_ports=( 67,  68), blocked_adj_split=4),
        _qsfp(label="31", adj_label=None,
              phy_addrs=(30, 30, 30, 30), hw_ports=( 69,  70,  71,  72)),
        _qsfp(label="32", adj_label=None,
              phy_addrs=(31, 31, 31, 31), hw_ports=( 71,  72), blocked_adj_split=4),
        _qsfp(label="33", adj_label=None,
              phy_addrs=(32, 32, 32, 32), hw_ports=(  5,   6,   7,   8)),
        _qsfp(label="34", adj_label=None,
              phy_addrs=(33, 33, 33, 33), hw_ports=(  7,   8), blocked_adj_split=4),
        _qsfp(label="35", adj_label=None,
              phy_addrs=(34, 34, 34, 34), hw_ports=(  1,   2,   3,   4)),
        _qsfp(label="36", adj_label=None,
              phy_addrs=(35, 35, 35, 35), hw_ports=(  3,   4), blocked_adj_split=4),
        _qsfp(label="37", adj_label=None,
              phy_addrs=(36, 36, 36, 36), hw_ports=( 13,  14,  15,  16)),
        _qsfp(label="38", adj_label=None,
              phy_addrs=(37, 37, 37, 37), hw_ports=( 15,  16), blocked_adj_split=4),
        _qsfp(label="39", adj_label=None,
              phy_addrs=(38, 38, 38, 38), hw_ports=(  9,  10,  11,  12)),
        _qsfp(label="40", adj_label=None,
              phy_addrs=(39, 39, 39, 39), hw_ports=( 11,  12), blocked_adj_split=4),
        _qsfp(label="41", adj_label=None,
              phy_addrs=(40, 40, 40, 40), hw_ports=( 21,  22,  23,  24)),
        _qsfp(label="42", adj_label=None,
              phy_addrs=(41, 41, 41, 41), hw_ports=( 23,  24), blocked_adj_split=4),
        _qsfp(label="43", adj_label=None,
              phy_addrs=(42, 42, 42, 42), hw_ports=( 17,  18,  19,  20)),
        _qsfp(label="44", adj_label=None,
              phy_addrs=(43, 43, 43, 43), hw_ports=( 19,  20), blocked_adj_split=4),
        _qsfp(label="45", adj_label=None,
              phy_addrs=(44, 44, 44, 44), hw_ports=( 29,  30,  31,  32)),
        _qsfp(label="46", adj_label=None,
              phy_addrs=(45, 45, 45, 45), hw_ports=( 31,  32), blocked_adj_split=4),
        _qsfp(label="47", adj_label=None,
              phy_addrs=(46, 46, 46, 46), hw_ports=( 25,  26,  27,  28)),
        _qsfp(label="48", adj_label=None,
              phy_addrs=(47, 47, 47, 47), hw_ports=( 27,  28), blocked_adj_split=4),
        _qsfp(label="49", adj_label=None,
              phy_addrs=(48, 48, 48, 48), hw_ports=( 53,  54,  55,  56)),
        _qsfp(label="50", adj_label=None,
              phy_addrs=(49, 49, 49, 49), hw_ports=( 55,  56), blocked_adj_split=4),
        _qsfp(label="51", adj_label=None,
              phy_addrs=(50, 50, 50, 50), hw_ports=( 49,  50,  51,  52)),
        _qsfp(label="52", adj_label=None,
              phy_addrs=(51, 51, 51, 51), hw_ports=( 51,  52), blocked_adj_split=4),
        _qsfp(label="53", adj_label=None,
              phy_addrs=(52, 52, 52, 52), hw_ports=( 61,  62,  63,  64)),
        _qsfp(label="54", adj_label=None,
              phy_addrs=(53, 53, 53, 53), hw_ports=( 63,  64), blocked_adj_split=4),
        _qsfp(label="55", adj_label=None,
              phy_addrs=(54, 54, 54, 54), hw_ports=( 57,  58,  59,  60)),
        _qsfp(label="56", adj_label=None,
              phy_addrs=(55, 55, 55, 55), hw_ports=( 59,  60), blocked_adj_split=4),
        _qsfp(label="57", adj_label=None,
              phy_addrs=(56, 56, 56, 56), hw_ports=( 37,  38,  39,  40)),
        _qsfp(label="58", adj_label=None,
              phy_addrs=(57, 57, 57, 57), hw_ports=( 39,  40), blocked_adj_split=4),
        _qsfp(label="59", adj_label=None,
              phy_addrs=(58, 58, 58, 58), hw_ports=( 33,  34,  35,  36)),
        _qsfp(label="60", adj_label=None,
              phy_addrs=(59, 59, 59, 59), hw_ports=( 35,  36), blocked_adj_split=4),
        _qsfp(label="61", adj_label=None,
              phy_addrs=(60, 60, 60, 60), hw_ports=( 45,  46,  47,  48)),
        _qsfp(label="62", adj_label=None,
              phy_addrs=(61, 61, 61, 61), hw_ports=( 47,  48), blocked_adj_split=4),
        _qsfp(label="63", adj_label=None,
              phy_addrs=(62, 62, 62, 62), hw_ports=( 41,  42,  43,  44)),
        _qsfp(label="64", adj_label=None,
              phy_addrs=(63, 63, 63, 63), hw_ports=( 43,  44), blocked_adj_split=4),
    )


class MlxSimxSN4600(cumulus.platform.Platform):
    name = 'mlnx,simx_msn4600'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSimxSN4600Switch(chip)
    pwms = None
    use_rate_api = True
    # SimX platforms do not simulate CPLD/LED/SFP/Fans/Sensors


Platform = MlxSimxSN4600
