#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation and its affiliates
# (the "Company") and all right, title, and interest in and to the software
# product, including all associated intellectual property rights, are and
# shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx

class NvidiaSN2201GigE(cumulus.platform.Spectrum1GPort):
    def __init__(self, label, adj_label, phy_addr, hw_port,
                 blocked_adj_split=None):

        hw_port_name = 'xe%d' % hw_port

        cumulus.platform.Spectrum1GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_port=hw_port_name,
                        phy_addr=phy_addr)

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class NvidiaSN2201QSFP28(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; NvidiaSN2201QSFP28
;;; Two bits per port: yellow(active high), green(active high)
'''
    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class NvidiaSN2201Switch(cumulus.platform.SpectrumSwitch):
    _gige = NvidiaSN2201GigE
    _qsfp = NvidiaSN2201QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _gige(label="1",   adj_label=None,  phy_addr=0,  hw_port=61),
        _gige(label="2",   adj_label=None,  phy_addr=1,  hw_port=62),
        _gige(label="3",   adj_label=None,  phy_addr=2,  hw_port=63),
        _gige(label="4",   adj_label=None,  phy_addr=3,  hw_port=64),
        _gige(label="5",   adj_label=None,  phy_addr=4,  hw_port=57),
        _gige(label="6",   adj_label=None,  phy_addr=5,  hw_port=58),
        _gige(label="7",   adj_label=None,  phy_addr=6,  hw_port=59),
        _gige(label="8",   adj_label=None,  phy_addr=7,  hw_port=60),
        _gige(label="9",   adj_label=None,  phy_addr=8,  hw_port=53),
        _gige(label="10",  adj_label=None,  phy_addr=9,  hw_port=54),
        _gige(label="11",  adj_label=None,  phy_addr=10, hw_port=55),
        _gige(label="12",  adj_label=None,  phy_addr=11, hw_port=56),
        _gige(label="13",  adj_label=None,  phy_addr=12, hw_port=49),
        _gige(label="14",  adj_label=None,  phy_addr=13, hw_port=50),
        _gige(label="15",  adj_label=None,  phy_addr=14, hw_port=51),
        _gige(label="16",  adj_label=None,  phy_addr=15, hw_port=52),
        _gige(label="17",  adj_label=None,  phy_addr=16, hw_port=45),
        _gige(label="18",  adj_label=None,  phy_addr=17, hw_port=46),
        _gige(label="19",  adj_label=None,  phy_addr=18, hw_port=47),
        _gige(label="20",  adj_label=None,  phy_addr=19, hw_port=48),
        _gige(label="21",  adj_label=None,  phy_addr=20, hw_port=41),
        _gige(label="22",  adj_label=None,  phy_addr=21, hw_port=42),
        _gige(label="23",  adj_label=None,  phy_addr=22, hw_port=43),
        _gige(label="24",  adj_label=None,  phy_addr=23, hw_port=44),
        _gige(label="25",  adj_label=None,  phy_addr=24, hw_port=37),
        _gige(label="26",  adj_label=None,  phy_addr=25, hw_port=38),
        _gige(label="27",  adj_label=None,  phy_addr=26, hw_port=39),
        _gige(label="28",  adj_label=None,  phy_addr=27, hw_port=40),
        _gige(label="29",  adj_label=None,  phy_addr=28, hw_port=33),
        _gige(label="30",  adj_label=None,  phy_addr=29, hw_port=34),
        _gige(label="31",  adj_label=None,  phy_addr=30, hw_port=35),
        _gige(label="32",  adj_label=None,  phy_addr=31, hw_port=36),
        _gige(label="33",  adj_label=None,  phy_addr=32, hw_port=1),
        _gige(label="34",  adj_label=None,  phy_addr=33, hw_port=2),
        _gige(label="35",  adj_label=None,  phy_addr=34, hw_port=3),
        _gige(label="36",  adj_label=None,  phy_addr=35, hw_port=4),
        _gige(label="37",  adj_label=None,  phy_addr=36, hw_port=5),
        _gige(label="38",  adj_label=None,  phy_addr=37, hw_port=6),
        _gige(label="39",  adj_label=None,  phy_addr=38, hw_port=17),
        _gige(label="40",  adj_label=None,  phy_addr=39, hw_port=18),
        _gige(label="41",  adj_label=None,  phy_addr=40, hw_port=19),
        _gige(label="42",  adj_label=None,  phy_addr=41, hw_port=20),
        _gige(label="43",  adj_label=None,  phy_addr=42, hw_port=21),
        _gige(label="44",  adj_label=None,  phy_addr=43, hw_port=22),
        _gige(label="45",  adj_label=None,  phy_addr=44, hw_port=23),
        _gige(label="46",  adj_label=None,  phy_addr=45, hw_port=24),
        _gige(label="47",  adj_label=None,  phy_addr=46, hw_port=25),
        _gige(label="48",  adj_label=None,  phy_addr=47, hw_port=26),

        _qsfp(label="49",  adj_label=None, phy_addrs=(48,48,48,48), hw_ports=( 9,10,11,12)),
        _qsfp(label="50",  adj_label=None, phy_addrs=(49,49,49,49), hw_ports=( 7, 8), blocked_adj_split=4),
        _qsfp(label="51",  adj_label=None, phy_addrs=(50,50,50,50), hw_ports=(13,14,15,16)),
        _qsfp(label="52",  adj_label=None, phy_addrs=(51,51,51,51), hw_ports=(11,12),blocked_adj_split=4),
    )

class NvidiaSN2201Fan(cumulus.sensors.Fan_Unit):
    fan_max = 19500
    fan_min = 2235

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

# System fan #1
class NvidiaSN2201Fan_1(NvidiaSN2201Fan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #2
class NvidiaSN2201Fan_2(NvidiaSN2201Fan):
    name = 'Fan2'
    description = 'Fan Tray 2'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #3
class NvidiaSN2201Fan_3(NvidiaSN2201Fan):
    name = 'Fan3'
    description = 'Fan Tray 3'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# System fan #4
class NvidiaSN2201Fan_4(NvidiaSN2201Fan):
    name = 'Fan4'
    description = 'Fan Tray 4'
    fan_num = name.lower()
    mlnx_hw_mgmt_fan_min = fan_num + '_min'
    mlnx_hw_mgmt_fan_max = fan_num + '_max'
    mlnx_hw_mgmt_fan_var = fan_num + '_var'
    mlnx_hw_mgmt_fan_speed = fan_num + '_speed_get'
    mlnx_hw_mgmt_fan_status = fan_num + '_status'
    mlnx_hw_mgmt_fan_fault = fan_num + '_fault'
    fan_attr_alias = fan_num + '_input'

# Base class for the Temp Sensors
class NvidiaSN2201Temp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 75
    temp_max = 105
    temp_crit = 120

    mlnx_hw_mgmt_path = '/run/hw-management/thermal/'

# System temp sensor #2
class NvidiaSN2201Temp_2(NvidiaSN2201Temp):
    name = 'Temp2'
    description = 'CPU Core Sensor 0'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core6'
    mlnx_hw_mgmt_temp_crit = 'cpu_core6_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core6_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

# System temp sensor #3
class NvidiaSN2201Temp_3(NvidiaSN2201Temp):
    name = 'Temp3'
    description = 'CPU Core Sensor 1'
    temp_num = name.lower()
    mlnx_hw_mgmt_temp_val = 'cpu_core12'
    mlnx_hw_mgmt_temp_crit = 'cpu_core12_crit'
    mlnx_hw_mgmt_temp_max = 'cpu_core12_max'
    temp_attr_prefix = name.lower()
    temp_attr_alias = temp_attr_prefix + '_input'

class NvidiaSN2201Sensors():

    def __init__(self):
        self.units = (NvidiaSN2201Fan_1(),
                      NvidiaSN2201Fan_2(),
                      NvidiaSN2201Fan_3(),
                      NvidiaSN2201Fan_4(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPSUFan_1(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPSUFan_2(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxTemp_1(),
                      NvidiaSN2201Temp_2(),
                      NvidiaSN2201Temp_3(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxTemp_4(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxTemp_5(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxTemp_6(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPSUTemp_1(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPSUTemp_2(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPSU1(),
                      cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPSU2(),
                      )

class NvidiaSN2201(cumulus.platform.Platform):
    name = 'nvidia,sn2201'
    chip = cumulus.platform.SpectrumChip()
    switch = NvidiaSN2201Switch(chip)
    cpld = cumulus.platforms.mlnx_SN2xxx.Mlx2xxxCPLD()
    sensors = NvidiaSN2201Sensors()
    pwms = None
    statusleds = cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLEDs()

Platform = NvidiaSN2201
