# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2014,2015,2016,2017,2018,2019,2020,2021  Cumulus Networks, Inc.

#-------------------------------------------------------------------------------
#
# Imports
#
import os
import re
import subprocess
import time
import cPickle
import imp
import exceptions
import syslog
import json

#-------------------------------------------------------------------------------
#
# Classes
#
class Unit:
    """
    Base Class all for all Sensors.

    name - Name of the sensor
    description - Description about the sensor
    attrs - Contains the attributes obtained from sensors and
            other useful info
    handlers - dict containing state and handler method.
    status_path - Path where sensors info is stored. Used by smonctl.
    """
    attrs = None
    handlers = None
    status_path = '/run/cache/cumulus/unit_state'
    hwmon_search = True
    dynamic_hwmon = False
    mlnx_hw_mgmt_path = None

    def __init__(self):
        self.attrs = dict()
        self.handlers = dict()
        if not hasattr(self, 'name'):
            self.attrs['name'] = self.__class__.__name__
        else:
            self.attrs['name'] = self.name
        if not hasattr(self, 'description'):
            self.attrs['description'] = self.attrs['name']
        else:
            self.attrs['description'] = self.description
        self.attrs['prev_state'] = 'UNKNOWN'
        self.attrs['state'] = 'UNKNOWN'
        self.attrs['msg'] = None
        self.attrs['prev_msg'] = None
        self.add_state('UNKNOWN', self.do_nothing)
        self.add_state('ABSENT', self.do_nothing)
        self.add_state('BAD', self.do_nothing)
        self.add_state('OK', self.do_nothing)

    def update_cpld_path(self, path):
        """
        Update the CPLD Path
        """
        self.cpld_path = path

    def update_cache(self):
        """
        Writes the sensor info at the status path.
        Default status path is: /run/cache/cumulus/unit_state
        """
        if not os.path.exists(self.status_path):
            os.makedirs(self.status_path)
        direct = os.path.join(self.status_path, self.attrs['name'])
        if not os.path.exists(direct):
            os.makedirs(direct)
        path = os.path.join(direct, 'state')
        try:
            f = open(path, 'w')
            f.write(self.attrs['state'] + '\n')
            f.close()
        except IOError, e:
            self.update_msg('Unable to update state at %s' %path)
        try:
            path = os.path.join(direct, 'dump')
            f = open(path, 'w')
            cPickle.dump(self.attrs, f)
            f.close()
        except IOError, e:
            self.update_msg('Unable to update dump at %s' %path)

    def add_state(self, name, handler):
        """
        Add state and handler in the handlers dict
        """
        self.handlers[name] = handler

    def set_state(self, name):
        """
        Update the state, previous state and reset time
        spent in a state
        """
        self.attrs['prev_state'] = self.attrs['state']
        self.attrs['state'] = name
        cur_st = self.attrs['state']
        prev_st = self.attrs['prev_state']
        if cur_st != prev_st:
            self.reset_time()

    def detect_state_plat(self):
        """
        First thing that runs in detect_state. This is the place
        to add platform specific checks.
        Default returns 'OK'
        """
        return 'OK'

    def detect_state(self):
        """
        Detect the state the sensor is in.
        Return state. Default possible states are: UNKNOWN, ABSENT, OK and BAD.
        Default state is UNKNOWN
        """
        return 'UNKNOWN'

    def override_defaults(self):
        """
        override_defaults runs in detect_state after the attributes/values have
        been collected. Use this method to change/manipulate values.
        """
        pass

    def run_state(self):
        """
        Detect the state and run its handler.
        Update sensors information at the cache path
        """
        self.clear_msg()
        st = self.detect_state()
        self.set_state(st)
        if self.handlers[st] is not None:
            self.handlers[st]()
        self.update_cache()

    def do_nothing(self):
        """
        As said, do nothing. Used as a handler
        """
        pass

    def update_status(self, status, attr):
        if status is None:
            status = attr
        else:
            status = status + ', %s' %attr
        return status

    def update_msg(self, msg):
        """
        Add msg to the self.attrs in the format:
        name(description): msg
        attrs['msg'] can be used is a display tool to provide
        more info.
        """
        if self.attrs['msg'] is None:
            self.attrs['msg'] = ''
        else:
            self.attrs['msg'] += '\n'
        self.attrs['msg'] += '%s' %self.attrs['name']
        if self.attrs['name'] != self.attrs['description']:
            self.attrs['msg'] += '(%s)' %self.attrs['description']
        self.attrs['msg'] += (':  ' + msg)

    def clear_msg(self):
        """
        Copy attrs['msg'] to attrs['prev_msg'] (prev_state msg).
        set attrs['msg'] to None
        """
        self.attrs['prev_msg'] = self.attrs['msg']
        self.attrs['msg'] = None

    def reset_time(self):
        """
        Update attrs,start_time and log_time, with current time.
        start_time indicates when you got into a state.
        log_time is the time when the last log msg was created.
        log_time is used in thresh_handler and crit_handler.
        """
        attrs = ['start_time', 'log_time']
        y = int(time.time())
        for x in attrs:
            self.attrs[x] = y

    def get_gpio_value(self, source):
        """
        Get the GPIO value from a given source. Path is:
        /sys/class/gpio/source/value
        """
        gpio_path = '/sys/class/gpio/'
        source += '/value'
        fn = os.path.join(gpio_path, source)
        try:
            value = open(fn, 'r').read()
        except:
            self.update_msg('Unable to update GPIO: %s' %source)
            return None
        return value.strip().rstrip('\0')

    def set_gpio_value(self, source, value):
        """
        Set the value at a given GPIO source. Path is:
        /sys/class/gpio/source/value
        """
        gpio_path = '/sys/class/gpio/'
        source += '/value'
        fn = os.path.join(gpio_path, source)
        try:
            fh = open(fn, 'w+')
            x = fh.write(value)
            fh.close()
        except:
            self.update_msg('Unable to update GPIO: %s' %source)
            return None
        return x

    def read_hw_mgmt_node(self, path, f, rettype, scale=1, attr_name=None):
        """
        Returns the value of  a Mellanox hw-mgmt node

        path    -   directory of the Mellanox hw-mgmt nodes of interest
        f       -   filename of the Mellanox hw-mgmt node of interest 
        rettype -   type (int/float/str) of the data to be returned to the caller
        scale   -   division factor (used when reading temp values/settings)
        attr_name - alternate name to name this quantity in the attribute list
        """
        val = 0
        err = 0
        type_val = None
 
        if attr_name is None:
            attr_name = f

        fname = os.path.join(path, f)
        if os.path.exists(fname) == False:
            syslog.syslog(syslog.LOG_WARNING, 'Device %s does not exist' % self.attrs['name'])
            self.update_msg('Device %s does not exist' %f)
            if (attr_name != 'skip'):
                self.attrs[attr_name] = None
            err |= -1
            return err, type_val

        retry_count = 5
        while retry_count > 0:
            try:
                with open(fname) as fp:
                    val =  fp.read().rsplit('\n')[0]
                    if rettype == 'int':
                        type_val = (int) (val)
                        type_val = type_val / scale
                    elif rettype == 'float':
                        type_val = (float) (val)
                        type_val = type_val / scale
                    elif rettype == 'string':
                        type_val = (str) (val)
                    else:
                        type_val = (str) (val)

                    if (attr_name != 'skip'):
                        self.attrs[attr_name] = type_val
                    break
            except:
                    retry_count = retry_count - 1
                    time.sleep(1)
                    continue

        if retry_count == 0:
            syslog.syslog(syslog.LOG_WARNING, 'Unable to read device %s' % self.attrs['name'])
            self.update_msg('Unable to read %s' %f)
            if (attr_name != 'skip'):
                self.attrs[attr_name] = None
            err |= -1

        return err, type_val


class PSU_Unit(Unit):
    """
    PSU System Unit.
    """

    def __init__(self):
        Unit.__init__(self)
        self.attrs['type'] = 'power'
 
    def detect_state(self):
        st = self.detect_state_plat()
        if st != 'OK':
            return st

        status = None
 
        # On switches with removeable PSUs, the psu_status node tells us is the PSU
        # is present or not.  There are a handful of systems that contained fixed
        # (not removable) PSUs.  For those systems, the hw-mgmt package does not
        # expose a psu_status node.  So we pretend psu_status is good.
        if self.mlnx_hw_mgmt_psu_status is None:
            psu_status = 1
        else:
            err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_psu_status, 'int')
            if err:
                status = self.update_status(status, 'not_ok')
                return 'BAD'
            psu_status = val

        # The psu_pwr_status tells us that the PSU is supplying power to the system.
        err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_psu_pwr_status, 'int')
        if err:
            status = self.update_status(status, 'not_ok')
            return 'BAD'
        psu_pwr_status= val

        # The normal scenario - PSU is inserted and is supplying power
        if (psu_status == 1) and (psu_pwr_status == 1):
            return 'OK'
        # PSU is inserted, but AC power is disconnected
        elif (psu_status == 1) and (psu_pwr_status == 0):
            self.update_msg('Not plugged in')
            return 'BAD'
        # PSU is not present
        elif (psu_status == 0) and (psu_pwr_status == 0):
            self.update_msg('Not installed')
            return 'ABSENT'
        # We should never get here
        else:
            status = self.update_status(status, 'not_ok')
            return 'BAD'


class Fan_Unit(Unit):
    """
    Fan Sensor Unit.

    fan_max - Max speed for the fan
    fan_max - Min speed for the fan
    fan_var - Variance of fan speed in %
    """

    fan_max = 29000
    fan_min = 2500
    fan_var = 15
    mlnx_hw_mgmt_psu_fan = False

    def __init__(self):
        Unit.__init__(self)
        self.attrs['type'] = 'fan'
        self.add_state('LOW', self.do_nothing)
        self.add_state('HIGH', self.do_nothing)

        self.attrs['device_path'] = self.mlnx_hw_mgmt_path + self.mlnx_hw_mgmt_fan_speed

    def override_defaults(self):
        # For PSU fans, the hw-mgmt package does exports min/max fan speeds,
        # but they are located in a different directory.
        if self.mlnx_hw_mgmt_psu_fan:
            err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_fan_min, 'int', attr_name=self.fan_min_attr_alias)
            if err == 0:
                self.fan_min = val

            err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_fan_max, 'int', attr_name=self.fan_max_attr_alias)
            if err == 0:
                self.fan_max = val
        else:
            err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_fan_min, 'int')
            if err == 0:
                self.fan_min = val

            err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_fan_max, 'int')
            if err == 0:
               self.fan_max = val

        # For fan variance, we must use hard-coded values
        self.attrs[self.mlnx_hw_mgmt_fan_var] = self.fan_var

    def detect_state(self):
        st = self.detect_state_plat()
        if st != 'OK':
            return st

        # For PSUs, the hw-mgmt package exports the fan speed only when
        # the PSU is inserted and is supplying power to the system.
        if self.mlnx_hw_mgmt_psu_fan:
            psu_state = True
            for psu_status in self.mlnx_hw_mgmt_psu_status_nodes:
                err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, psu_status, 'int', attr_name='skip')
                if err:
                    val = 0
                if val == 0:
                    psu_state = False

            if psu_state == False:
                return 'ABSENT'
 
        self.override_defaults()

        status = None
 
        err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_fan_speed, 'int', attr_name=self.fan_attr_alias)
        if err:
            status = self.update_status(status, 'not_ok')
            return 'BAD'
        fan_speed = val

        if self.mlnx_hw_mgmt_psu_fan == True:
            # The Mellanox hw-mgmt pkg does not export nodes for PSU fan_status or PSU fan_fault
            # In order to keep the code generic, we pretend that all is well here.
            fan_status = 1
            fan_fault = 0
        else:
            err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_fan_status, 'int')
            if err:
                status = self.update_status(status, 'not_ok')
                return 'BAD'
            fan_status = val

            if self.mlnx_hw_mgmt_fan_fault is not None:
                err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_fan_fault, 'int')
                if err:
                    status = self.update_status(status, 'not_ok')
                    return 'BAD'
                fan_fault = val
            else:
                if self.mlnx_hw_mgmt_fan_ok is not None:
                    # This block of code exists solely for the VX image.  The VX image reports
                    # fan fault different from the Mellanox hw-mgmt package.
                    err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_fan_ok, 'int')
                    if err:
                        status = self.update_status(status, 'not_ok')
                        return 'BAD'
                    if val:
                        fan_fault = 0   
                    else:
                        fan_fault = 1   

        # Fan is present and no faults are detected (the normal scenario)
        if (fan_status == 1) and (fan_fault == 0):
            # Fan is present but is not spinning!!! )
            if (fan_speed == 0):
                self.update_msg('Fan speed is at 0 RPM (not working or absent)')
                return 'ABSENT'
            # Fan is present but is spinning too slow!!!
            elif (fan_speed <= (self.fan_min * (1 - (float(self.fan_var)/100)))):
                self.update_msg('Fan input %d RPM is lower than %d RPM' %(fan_speed, self.fan_min))
                return 'LOW'
            # Fan is present but is spinning too high
            elif (fan_speed >= (self.fan_max * (1 + (float(self.fan_var)/100)))):
                self.update_msg('Fan speed %d RPM greater than %d RPM' %(fan_speed, self.fan_max))
                return 'HIGH'
            else:
                return 'OK'
        # Fan is not present!!!
        elif (fan_status == 0):
            self.update_msg('Not installed')
            return 'ABSENT'
        # Fan fault has been detected!!!
        elif (fan_fault == 1):
            status = self.update_status(status, 'fault')
            return 'BAD'
        # We should never get here
        else:
            status = self.update_status(status, 'not_ok')
            return 'BAD'


class Temp_Unit(Unit):
    """
    Temp Sensor Unit.

    Possible states: OK, BAD, UNKNOWN, ABSENT, HIGH, LOW, CRITICAL, LCRITICAL
    temp_crit = Critical Temp
    temp_max = Max Temp
    temp_fan_min = Temp at which corresponding fan speed is minimum
    temp_fan_max = Temp at which corresponding fan speed is maximum
    temp_min = Min Temp
    temp_lcrit = Low Crit Temp
    thres_log_interval = Log message and run if Temp is in HIGH/LOW/LCRITICAL state
                         every thres_log_interval secs
    crit_log_interval = Log message if Temp is in CRITICAL state every
                        crit_log_interval secs
    crit_shutdown_time = Shutdown system if sensor is in CRITICAL  state
                         for crit_shutdown_time secs
    """

    temp_crit = 85
    temp_max = 80
    temp_fan_min = 25
    temp_min = 5
    temp_lcrit = 0
    dynamic_limits = False
    thres_log_interval = 300
    crit_log_interval = 10
    crit_shutdown_time = 60
    no_shutdown = False
    hist_max_readings = 50
    hist_avg_count = 10
    temp_oper_max = 150
    temp_oper_min = -50
    # Only flag the sensor as BAD if you see 'temp_oper_cnt' outliers
    temp_oper_cnt = 3
    temp_otp_hw = False
    # Put syslog messages when HW OTP is enabled and temp is (crit - otp_hw_offset)
    temp_otp_hw_offset = 4
    mlnx_hw_mgmt_psu_temp = False

    def __init__(self):
        Unit.__init__(self)

        self.attrs['type'] = 'temp'

        if not hasattr(self, 'temp_fan_max'):
            self.temp_fan_max = self.temp_max
        if not hasattr(self, 'no_fans'):
            self.no_fans = False 

        self.attrs['input_hist'] = []
        self.attrs['start_time'] = None
        self.attrs['log_time'] = None
        self.add_state('HIGH', self.thres_handler)
        self.add_state('LOW', self.thres_handler)
        self.add_state('CRITICAL', self.crit_handler)
        # In every case so far, we have seen temp sensors report LCRITICAL because
        # of a broken sensor or SMBUS reporting -ve error. Hence, we will not
        # be shutting down the switch on LCRITICAL, but instead log warnings.
        # Since something is broken, we may not be able to detect CRITICAL
        # temperatures too. So play safe, and set the fan speed to max.
        self.add_state('LCRITICAL', self.thres_handler)
        # Add your input manipulation function here. Used to find right state.
        self.state_input_func = self.weighted_average

        self.attrs['device_path'] = self.mlnx_hw_mgmt_path + self.mlnx_hw_mgmt_temp_val

    def override_defaults(self):
        if self.mlnx_hw_mgmt_temp_max is not None:
            err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_temp_max, 'float', scale=1000, attr_name='skip')
            if err == 0:
                self.temp_max = val

        if self.mlnx_hw_mgmt_temp_crit is not None:
            err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_temp_crit, 'float', scale=1000, attr_name='skip')
            if err == 0:
                self.temp_crit = val

        p = self.temp_attr_prefix
        self.attrs[p + '_max'] = self.temp_max
        self.attrs[p + '_min'] = self.temp_min
        self.attrs[p + '_crit'] = self.temp_crit
        self.attrs[p + '_lcrit'] = self.temp_lcrit
        if self.no_fans == False:
            self.attrs[p + '_fan_max'] = self.temp_fan_max
            self.attrs[p + '_fan_min'] = self.temp_fan_min      

    def weighted_average(self):
        weights = [0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.11, 0.16, 0.21, 0.28]
        count = len(self.attrs['input_hist'])

        if count > self.hist_avg_count:
            count = self.hist_avg_count
            last_n_values = self.attrs['input_hist'][-count:]
            self.attrs['input_avg'] = 0
            for i in range(0,count):
                self.attrs['input_avg'] += weights[i] * last_n_values[i]
        else:
            last_n_values = self.attrs['input_hist']
            self.attrs['input_avg'] = sum(self.attrs['input_hist'])/count
        return self.attrs['input_avg']

    def find_outliers(self):
        """
        Some temperature sensors can rarely report incorrect values.
        If a value is outside the valid range, report the invalid value
        and ignore it.
        """
        count = len(self.attrs['input_hist'])
        found = []

        if count > self.hist_avg_count:
            count = self.hist_avg_count
            last_n_values = self.attrs['input_hist'][-count:]
        else:
            last_n_values = self.attrs['input_hist']
        found += (i for i in last_n_values if i > self.temp_oper_max)
        found += (i for i in last_n_values if i < self.temp_oper_min)
        return found

    def has_zeroes(self):
        """
        Sometimes one sensor will consistently show 0 C, even though the rest
        of the sensors on the same chip are showing normal temperatures.
        So the 0 C case must be dealt as a special case and we should not
        shutdown the box, but rather log this a bad sensor. 
        """
        hist_count = len(self.attrs['input_hist'])
        check_count = self.hist_avg_count/2
        check_list = self.attrs['input_hist'][-hist_count:]
        if hist_count < self.hist_avg_count:
            check_count = int(hist_count/2) + 1

        if check_list.count(0) >= check_count:
            self.update_msg('More than half the last 10 readings are showing zeroes')
            return True

        return False

    def update_input_history(self, val):
        if not self.attrs.has_key('input_hist'):
            self.attrs['input_hist'] = []
        if len(self.attrs['input_hist']) > self.hist_max_readings:
            self.attrs['input_hist'].pop(0)
        self.attrs['input_hist'].append(val)

    def update_crit_action_path(self, path):
        """
        Sets crit_action_path which will point to the script which will run when 
        the unit status is CRTITICAL.
        """
        self.attrs['crit_action_path'] = path

    def update_thres_action_path(self, path):
        """
        Sets crit_action_path which will point to the script which will run when 
        the unit status is HIGH or LOW.
        """
        self.attrs['thres_action_path'] = path

    def check_otp(self, inp):
        temp_otp_warn = self.temp_crit - self.temp_otp_hw_offset
        if self.temp_otp_hw and (inp >= temp_otp_warn):
            self.update_msg('Temperature (%d C) is approaching thermal hardware shutdown (%d C)' \
                            %(inp, self.temp_crit))

    def detect_state(self):
        st = self.detect_state_plat()
        if st != 'OK':
            return st

        # For PSUs, the hw-mgmt package exports the fan speed only when
        # the PSU is inserted and is supplying power to the system.
        if self.mlnx_hw_mgmt_psu_temp:
            psu_state = True
            for psu_status in self.mlnx_hw_mgmt_psu_status_nodes:
                err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, psu_status, 'int', attr_name='skip')
                if err:
                    val = 0
                if val == 0:
                    psu_state = False

            if psu_state == False:
                return 'ABSENT'

        self.override_defaults()

        err, val = self.read_hw_mgmt_node(self.mlnx_hw_mgmt_path, self.mlnx_hw_mgmt_temp_val, 'float', scale=1000, attr_name=self.temp_attr_alias)
        if err:
            return 'BAD'
        if val is None:
            self.update_msg('Sensor has no value to read')
            return 'BAD'
     
        self.update_input_history(val)
        val = self.state_input_func()

        if val >= self.temp_crit:
            found = self.find_outliers()
            if len(found) > 0:
                # Only complain if 3/10 entries are outliers.
                if len(found) >= self.temp_oper_cnt:
                    self.update_msg("Following outliers were found: %s C" %found)
                    return 'BAD'
                # If there is a random outlier, ignore for now.
                else:
                    return self.attrs['prev_state']
            self.check_otp(val)
            return 'CRITICAL'
        elif val >= self.temp_max:
            self.check_otp(val)
            return 'HIGH'
        elif val <= self.temp_lcrit:
            if self.has_zeroes():
                return 'BAD'
            found = self.find_outliers()
            if len(found) > 0:
                self.update_msg("Following outliers were found: %s C" %found)
                return 'BAD'
            return 'LCRITICAL'
        elif val <= self.temp_min:
            return 'LOW'

        return 'OK'

    def thres_handler(self):
        """
        This is the handler when the state is HIGH or LOW.
        time_from_last_log indicates when the last log message was updated.
        time_in_state indicates how long unit has been in the state.
        This modules updates the msg attribute based on thres_log_interval,
        time_in_state and current time.
        Every time we in this handler, we run the script pointed by
        thres_action_path.
        """
        self.log_thres_state()
        if self.attrs['thres_action_path'] is not None:
            subprocess.call(self.attrs['thres_action_path'])

    def crit_handler(self):
        """
        This is the handler when the state is CRITICAL.
        time_from_last_log indicates when the last log message was updated.
        time_in_state indicates how long unit has been in the state.
        time_to_shutdown is time left for shutdown action.
        This modules updates the msg attribute based on crit_log_interval,
        time_in_state, time_to_shutdown and current time.
        When time_in_state equals time_to_shutdown, shutdown the system.
        Every time we are in this handler, we run the script pointed by
        crit_action_path.
        """
        if self.no_shutdown:
            self.log_thres_state()
            return

        cur_t = int(time.time())
        log_t = self.attrs['log_time']
        start_t = self.attrs['start_time']
        time_from_last_log = cur_t - log_t
        time_in_state = cur_t - start_t
        time_to_shutdown = self.crit_shutdown_time - time_in_state

        if time_to_shutdown <= 0:
            msg = ('Avg state is %s for last %s secs. Last %d values: %s. Generating cl-support and shutting down system...' \
                            %(self.attrs['state'], self.crit_shutdown_time, \
                            self.hist_avg_count, self.attrs['input_hist'][-self.hist_avg_count:]))
            self.update_msg(msg)
            self.update_cache()
            w = subprocess.Popen('/usr/bin/wall', stdin=subprocess.PIPE)
            w.communicate(input=msg);
            w.stdin.close()
            w.wait()
            subprocess.call('/usr/cumulus/bin/cl-support')
            subprocess.call("/sbin/poweroff")
        else:
            if time_from_last_log >= self.crit_log_interval:
                self.attrs['log_time'] = cur_t
                msg = ('Avg state is %s. Last %d values: %s. System will shutdown in %d secs' \
                            %(self.attrs['state'], self.hist_avg_count, \
                              self.attrs['input_hist'][-self.hist_avg_count:], time_to_shutdown))
                self.update_msg(msg)
                w = subprocess.Popen('/usr/bin/wall', stdin=subprocess.PIPE)
                w.communicate(input=msg);
                w.stdin.close()
                w.wait()
        if self.attrs['crit_action_path'] is not None:
            subprocess.call(self.attrs['crit_action_path'])

    def log_thres_state(self):
        log_t = self.attrs['log_time']
        start_t = self.attrs['start_time']
        cur_t = int(time.time())
        time_from_last_log = cur_t - log_t
        time_in_state = cur_t - start_t
        temp = None
        for key, value in self.attrs.iteritems():
            if re.match('temp[0-9]*_input', key):
                temp = int(value)
        if time_from_last_log >= self.thres_log_interval:
            self.attrs['log_time'] = cur_t
            self.update_msg('Temperature(%d C) is %s for last %s secs' \
                            %(temp, self.attrs['state'],time_in_state))
        else:
            self.update_msg('Temperature(%d C) is %s for last %s secs' \
                            %(temp, self.attrs['state'],time_in_state))
            self.attrs['prev_msg'] = self.attrs['msg']


class StatusLED_Unit():
    """
    Status LED Unit.
    Manages Status LED present on the front panel

    sources - Indicates the sensors to look at. Sensors state is obtained from
              /run/cache/cumulus/unit_state/source/state
              If None, look at all the the sensors in unit_state.
    good_states - If all sources, have good_state, set good_led_color.
    good_led_color - If all sources, have good_state, set good_led_color,
                     default is green.
    bad_states - If any of the sources, have bad_state, set bad_led_color.
    bad_led_color - If any of the sources, have bad_state, set bad_led_color,
                    default is yellow.
    led_path  - Location of the LED, if not specified, use CPLD path.
    led_location - Led is at led_path/led_location.
    """
    sources = None
    good_states = ('OK',)
    bad_states = ('BAD', 'UNKNOWN', 'ABSENT', 'HIGH', 'LOW', 'CRITICAL', 'LCRITICAL')
    good_led_color = 'green'
    bad_led_color = 'yellow'
    attrs = None
    status_path = '/run/cache/cumulus/unit_state'
    override_gpio_path = False

    def __init__(self):

        self.attrs = dict()
        self.handlers = dict()
        if not hasattr(self, 'name'):
            self.attrs['name'] = self.__class__.__name__
        else:
            self.attrs['name'] = self.name
        if not hasattr(self, 'description'):
            self.attrs['description'] = self.attrs['name']
        else:
            self.attrs['description'] = self.description

        self.attrs['sources'] = self.sources
        self.attrs['good_led_color'] = self.good_led_color
        self.attrs['bad_led_color'] = self.bad_led_color
        if hasattr(self, 'led_path'):
            self.attrs['led_path'] = self.led_path
        if hasattr(self, 'led_location'):
            self.attrs['led_location'] = self.led_location
        if hasattr(self, 'override_gpio_path'):
            self.attrs['override_gpio_path'] = self.override_gpio_path

    def update_msg(self, msg):
        syslog.syslog(syslog.LOG_WARNING, msg)

    def update_cpld_path(self, path):
        """
        Update the CPLD Path as led_path
        """
        if not hasattr(self, 'led_path'):
            self.led_path = path
            self.attrs['led_path'] = self.led_path

    def led_update_status(self):
        """
        Return a dict of source and state.
        """
        status = dict()

        if os.path.exists(self.status_path):
            try:
                files = os.listdir(self.status_path)
            except OSError, e:
                return status
        else:
            return status

        if self.sources is None:
            self.sources = files
            self.attrs['sources'] = self.sources

        for x in self.sources:
            if x in files:
                path = os.path.join(self.status_path, x, 'state')
                # ledmgrd can reach here before smond has created the status files for sensors.
                # In such a case, assume the sensor is absent, and move on, instead of crashing.
                try:
                    fp = open(path, 'r')
                    status[x] = fp.read().strip('\n')
                    fp.close()
                except IOError, e:
                    status[x] = 'ABSENT'
        self.attrs['status'] = status
        return status

    def led_find_color(self, status):
        """
        If any of the sources has bad_state, return bad_led_color.
        Else return good_led_color.
        """
        if self.sources is None:
            return self.bad_led_color

        color = self.good_led_color
        for source in self.sources:
            if status.has_key(source):
                state = status[source]
                if state in self.bad_states:
                    color = self.bad_led_color
                    break
        return color

    def led_update(self, color):
        """
        Update the led color at  led_path/led_location
        """
        try:
            path = os.path.join(self.led_path, self.led_location)
            fp = open(path, 'w')
            fp.write(color)
            fp.close()
        except:
            self.update_msg('Cannot update led: %s' %self.led_location)

    def led_read(self):
        """
        Read the led at led_path/led_location
        """
        try:
            path = os.path.join(self.led_path, self.led_location)
            fp = open(path, 'r')
            color = fp.read().strip('\n')
            fp.close()
        except:
            self.update_msg('Cannot update led: %s' %self.led_location)
            color = None
        return self.attrs['name'], color

    def run_state(self):
        """
        From the dict of (source, state), find the led color and
        update it.
        """
        if not os.path.exists(self.status_path):
            # Dont update till smond writes to status_path
            return
        status = self.led_update_status()
        color = self.led_find_color(status)
        self.attrs['color'] = color
        self.attrs['status'] = status
        self.led_update(color)

    def get_gpio_value(self, source):
        """
        Get the GPIO value from a given source. Path is:
        /sys/class/gpio/source/value
        """
        if self.override_gpio_path is False:
            gpio_path = '/sys/class/gpio/'
            source += '/value'
        else:
            gpio_path = self.led_path

        fn = os.path.join(gpio_path, source)
        try:
            value = open(fn, 'r').read()
        except:
            self.update_msg('Unable to read GPIO: %s' %source)
            return None

        return value.strip().rstrip('\0')

    def set_gpio_value(self, source, value):
        """
        Set the value at a given GPIO source. Path is:
        /sys/class/gpio/source/value
        """
        if self.override_gpio_path is False:
            gpio_path = '/sys/class/gpio/'
            source += '/value'
        else:
            gpio_path = self.led_path
        try:
            fn = os.path.join(gpio_path, source)
            fh = open(fn, 'w+')
            x = fh.write(value)
            fh.close()
        except:
            self.update_msg('Unable to update GPIO: %s' %source)
            return None
        return x


dynamic_sensors_file = "/run/cumulus/platform/dynamic_sensors.py"

def dynamic_sensors():
    try:
        m = imp.load_source("dynamic_sensors", dynamic_sensors_file)
    except Exception:
        return ()
    return m.sensors
