#! /usr/bin/python

# Copyright (C) 2020-2021 NVIDIA Corporation. ALL RIGHTS RESERVED.
# Copyright 2020 Cumulus Networks, Inc.
# All rights reserved

'''
MSN4800 EEPROM
'''
#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import os
    import binascii
    import eeprom_base
    import eeprom_psu
    import eeprom_tlvinfo
except ImportError, misc_ex:
    raise ImportError(str(misc_ex) + "- required module not found")

#
# base board
#
class board(eeprom_tlvinfo.TlvInfoDecoder):
    '''
    SN4800 main EEPROM
    '''
    EEPROM_MAX_SIZE = 0x400
    EEPROM_OFFSET = 0x0

    def __init__(self, name, path, cpld_root, ro):
        super(board, self).__init__(path,
                                    self.EEPROM_OFFSET,
                                    "",
                                    ro,
                                    self.EEPROM_MAX_SIZE)

    def switchaddrstr(self, mac):
        '''
        Extract base MAC address
        '''
        return self.base_mac_addr(mac)



#
# power supply unit
#
class psu(eeprom_psu.eeprom_psu):
    '''
    SN4800 PSU EEPROM
    '''
    def check_status(self):
        '''
        Check the PSU presence
        '''
        try:
            path = os.path.join('/run/hw-management/thermal/psu%s_pwr_status' % self.name[3])
            with open(path, "r") as file_handle:
                status = file_handle.readline().rstrip()
            if int(status):
                return 'ok'
        except:
            pass
        return 'absent'

#
# Fan eeprom
#
class fan(eeprom_base.eepromDecoder):
    '''
    SN4800 Fan EEPROM
    '''
    fan_fmt = [('burn', 'h', 8),
               ('brand', 's', 4),
               ('burn', 'h', 28),
               ('part_num', 's', 12),
               ('checksum', 'c', 2)
              ]

    def __init__(self, name, path, cpld_root, ro):
        self.name = name
        super(fan, self).__init__(path,
                                  self.fan_fmt,
                                  0,
                                  cpld_root,
                                  ro)

    def decoder(self, key, val):
        '''
        Decode based eeprom format defined above
        '''
        if key == 'header':
            i = "0x" + binascii.b2a_hex(val).upper()
        else:
            i = val
        return i

    def check_status(self):
        '''
        Check device presence
        '''
        path = '/run/hw-management/thermal/fan%s_status' % self.name[3]
        try:
            with open(path, "r") as file_handle:
                status = file_handle.readline().strip()
                if status == '1':
                    return 'ok'
        except:
            pass
        return 'absent'

    def is_checksum_valid(self, csum):
        return (True, 0)
